package com.utopiaindustries.dialect;

import com.utopiaindustries.util.CTPDateTimeFormat;
import org.thymeleaf.IEngineConfiguration;
import org.thymeleaf.context.ITemplateContext;
import org.thymeleaf.engine.AttributeName;
import org.thymeleaf.model.IProcessableElementTag;
import org.thymeleaf.processor.element.AbstractAttributeTagProcessor;
import org.thymeleaf.processor.element.IElementTagStructureHandler;
import org.thymeleaf.standard.expression.IStandardExpression;
import org.thymeleaf.standard.expression.IStandardExpressionParser;
import org.thymeleaf.standard.expression.StandardExpressions;
import org.thymeleaf.templatemode.TemplateMode;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;

public class FormatDateTagProcessor extends AbstractAttributeTagProcessor {
	private static final String ATTR_NAME = "formatdate";
	private static final int PRECEDENCE = 100000;

	public FormatDateTagProcessor( final String dialectPrefix ) {
		super(
				TemplateMode.HTML,
				dialectPrefix,
				null,
				false,
				ATTR_NAME,
				true,
				PRECEDENCE,
				true
		);
	}

	protected void doProcess( ITemplateContext iTemplateContext, IProcessableElementTag iProcessableElementTag, AttributeName attributeName, String attributeValue, IElementTagStructureHandler iElementTagStructureHandler ) {
		final IEngineConfiguration configuration = iTemplateContext.getConfiguration();
		final IStandardExpressionParser parser = StandardExpressions.getExpressionParser( configuration );
		final IStandardExpression expression = parser.parseExpression( iTemplateContext, attributeValue );
		final LocalDate attributeValueParsed = ( LocalDate ) expression.execute( iTemplateContext );
		iElementTagStructureHandler.setBody( "", false );
		if( attributeValueParsed != null ){
			// format zoned date time
			String formattedDateTime = attributeValueParsed.format( DateTimeFormatter.ofPattern( CTPDateTimeFormat.TEMPLATE_DATE_FORMAT ) );
			// render the body
			iElementTagStructureHandler.setBody( formattedDateTime, false );
		}
	}
}
